// MyTreeView.cpp : implementation of the CMyTreeView class
//

#include "stdafx.h"
#include "MyTree.h"

#include "MyTreeDoc.h"
#include "MyTreeView.h"
#include "randdlg.h"
#include "inputdlg.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CMyTreeView

IMPLEMENT_DYNCREATE(CMyTreeView, CScrollView)

BEGIN_MESSAGE_MAP(CMyTreeView, CScrollView)
	//{{AFX_MSG_MAP(CMyTreeView)
	ON_COMMAND(ID_FILE_RADNOM, OnFileRadnom)
	ON_WM_MOUSEMOVE()
	ON_WM_LBUTTONDOWN()
	ON_COMMAND(ID_FILE_INPUT, OnFileInput)
	ON_COMMAND(ID_FILE_OPEN, OnFileOpen)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CMyTreeView construction/destruction

CMyTreeView::CMyTreeView()
{
	m_isEmpty=TRUE;
	m_text=0;
	MemoryInit();
}

CMyTreeView::~CMyTreeView()
{
	free(m_text);
	ClearTree();
}

BOOL CMyTreeView::PreCreateWindow(CREATESTRUCT& cs)
{
	return CScrollView::PreCreateWindow(cs);
}

/////////////////////////////////////////////////////////////////////////////
// CMyTreeView drawing

void CMyTreeView::OnDraw(CDC* pDC)
{
	CSize size;
	if(m_isEmpty) {pDC->TextOut(10,10,"[Empty]");return;}
	size=m_draw.Draw(pDC);
	size.cx+=16;
	size.cy+=16;
	SetScrollSizes(MM_TEXT,size);
}

void CMyTreeView::OnInitialUpdate()
{
	CScrollView::OnInitialUpdate();

	CSize sizeTotal;
	sizeTotal.cx = sizeTotal.cy = 100;
	SetScrollSizes(MM_TEXT, sizeTotal);
}

/////////////////////////////////////////////////////////////////////////////
// CMyTreeView diagnostics

#ifdef _DEBUG
void CMyTreeView::AssertValid() const
{
	CScrollView::AssertValid();
}

void CMyTreeView::Dump(CDumpContext& dc) const
{
	CScrollView::Dump(dc);
}

CMyTreeDoc* CMyTreeView::GetDocument() // non-debug version is inline
{
	ASSERT(m_pDocument->IsKindOf(RUNTIME_CLASS(CMyTreeDoc)));
	return (CMyTreeDoc*)m_pDocument;
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CMyTreeView message handlers

void CMyTreeView::OnFileRadnom() 
{
	int n;
	CRandDlg dlg;
	if(dlg.DoModal()!=IDOK) return;
	free(m_text); ClearTree();
	m_text=(char*)malloc(dlg.m_size+1);
	switch(dlg.m_type) {
	case 0: 
		for(n=0; n<dlg.m_size; n++) m_text[n]='a'+rand()%('z'-'a'+1);
		break;
	case 1:
		for(n=0; n<dlg.m_size; n++) m_text[n]='0'+rand()%10;
		break;
	case 2:
		for(n=0; n<dlg.m_size; n++) m_text[n]='!'+rand()%(127-'!');
		break;
	}
	m_text[n]=0;
	Generate();
}




void CMyTreeView::Generate()
{
	CString info;
	int len; clock_t t1,t2;
	len=strlen(m_text);
	t1=clock();
	m_root=MakeTree(m_text);
	t2=clock();
	CDC *pDC=GetDC();
	m_draw.SetRoot(pDC,m_root->links,m_text+len);
	ReleaseDC(pDC);
	m_isEmpty=FALSE; Invalidate();
	info.Format("Text size: %d\n" "Work time: %dms\n",len,t2-t1);
	AfxMessageBox(info,MB_ICONASTERISK);
}




void CMyTreeView::OnMouseMove(UINT nFlags, CPoint point) 
{
	point+=GetScrollPosition();
	if(m_draw.Mouse(point)) Invalidate(FALSE);
}

void CMyTreeView::OnLButtonDown(UINT nFlags, CPoint point) 
{
	CDC *pDC=GetDC();
	m_draw.Click(pDC);
	ReleaseDC(pDC);
	Invalidate();
}

void CMyTreeView::OnFileInput() 
{
	CInputDlg dlg;
	if(dlg.DoModal()!=IDOK) return;
	free(m_text); ClearTree();
	m_text=strdup(dlg.m_text.GetBuffer(0));
	Generate();
}

void CMyTreeView::OnFileOpen() 
{
	int n,m,size;
	CFileDialog dlg(TRUE,0,0,0,"All|*.*|Txt,Html|*.txt;*.htm*||");
	if(dlg.DoModal()!=IDOK) return;
	FILE *file=fopen(dlg.GetFileName(),"rb");
	if(!file) {AfxMessageBox("IO error"); return;}
	fseek(file,0,SEEK_END);
	size=ftell(file);
	fseek(file,0,SEEK_SET);
	free(m_text); ClearTree();
	m_text=(char*)malloc(size+1);
	size=fread(m_text,1,size,file);	
	fclose(file);
	for(n=m=0; n<size; n++) {
		if(m_text[n]==' ') m_text[n]='_';
		m_text[m]=m_text[n];
		if((BYTE)m_text[m]>' ') m++;
	}
	size=m; m_text[size]=0;
	if(!size) AfxMessageBox("Empty file");
	else Generate();
}
